@@ -163,10 +163,11 @@ from typing import Any, Dict, List, Optional, Sequence
 
 from ae.inspector import try_eval                                               # type: ignore
 from ae.core import DEBUG_LEVEL_VERBOSE                                         # type: ignore
+from ae.i18n import get_f_string                                                # type: ignore
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.1.17'
+__version__ = '0.1.18'
 
 
 NODE_FILE_PREFIX = 'node_'  #: file name prefix for node imports/exports
@@ -235,6 +236,19 @@ class LiszDataMixin(ABC):
 
     # helper methods
 
+    def add_item(self, nid: LiszItem) -> str:
+        """ add item to currently displayed node.
+
+        :param nid:             LiszItem to add (has to have a non-empty item id).
+        :return:                error message if any error happened, else empty string.
+        """
+        item_id = nid['id']
+        want_node = 'node' in nid
+        err_msg = self.edit_validate(-1, item_id, want_node=want_node)
+        if not err_msg and want_node:
+            self.current_node_items[0]['node'].extend(nid['node'])
+        return err_msg
+
     def change_sub_node_sel(self, node: LiszNode, set_sel_to: bool):
         """ change the selection of all the sub-leaves of the passed node to the specified value.
 
@@ -248,7 +262,23 @@ class LiszDataMixin(ABC):
             elif set_sel_to:
                 item['sel'] = 1.0
 
-    def delete_item(self, item_id: str, node_only: bool):
+    def current_item_or_node_repr(self) -> str:
+        """ return the currently focused/displayed item or node as repr string.
+
+        :return:                repr string of the currently focused item id/node or of the displayed node.
+        """
+        flow_id = self.flow_id
+        if flow_action(flow_id) != 'focus':
+            rpr = repr(self.flow_path_node())
+        else:
+            item = self.item_by_id(flow_key(flow_id))
+            if 'node' in item:
+                rpr = repr(item)
+            else:
+                rpr = item['id']
+        return rpr
+
+    def delete_item(self, item_id: str, node_only: bool = False):
         """ delete either the complete item or the sub node of the item (identified by the passed item id).
 
         :param item_id:         item id to identify the item/sub-node to be deleted.
@@ -260,9 +290,6 @@ class LiszDataMixin(ABC):
         else:
             assert nid in self.current_node_items, f"DEL item data: {nid} not in {self.current_node_items}"
             self.current_node_items.remove(nid)
-            self.change_flow(id_of_flow('', ''))
-
-        self.play_sound('deleted')
 
     def edit_validate(self, old_idx: int, new_id: Optional[str] = None, want_node: Optional[bool] = None) -> str:
         """ validate the user changes after adding a new item or editing an existing item.
@@ -279,14 +306,15 @@ class LiszDataMixin(ABC):
         add_item = old_idx == -1
         if new_id is not None:
             if not new_id:
-                # on empty id cancel edit if add_item, else request confirmation from user for item del
+                # on empty id string cancel edit if add_item, else request confirmation from user for item del
                 return "" if add_item else 'request_delete_confirmation_for_item'
             if not check_item_id(new_id):
-                return f"invalid item id {new_id} containing '{FLOW_PATH_TEXT_SEP}' or is equal '{FLOW_PATH_ROOT_ID}'"
+                return get_f_string(
+                    "invalid item id {new_id} containing '{FLOW_PATH_TEXT_SEP}' or is equal '{FLOW_PATH_ROOT_ID}'")
 
             new_idx = self.find_item_index(new_id)
             if new_idx != -1 and (add_item or new_idx != old_idx):
-                return f"item id '{new_id}' exists already"
+                return get_f_string("item id '{new_id}' exists already")
 
         if add_item:                        # add new item
             nid = dict(id=new_id)
@@ -436,7 +464,8 @@ class LiszDataMixin(ABC):
 
         :param flow_path:       flow path list.
         :param min_len:         minimum length of node ids (flow id keys).
-        :return:                shortened display text from the passed flow path.
+        :return:                shortened display text string of the passed flow path (which can be converted back
+                                to a flow path list with the method :meth:`.flow_path_from_text`).
         """
         path_nid = None         # suppress Pycharm PyUnboundLocalVariable inspection warning
         shortened_ids = list()
@@ -585,7 +614,11 @@ class LiszDataMixin(ABC):
         :param key_code:        key code string.
         :return:                True if key event got processed/used, else False.
         """
-        if key_code == 'r':
+        if modifiers == 'Ctrl' and key_code in ('c', 'v', 'x'):
+            if self.call_method('on_clipboard_key_' + key_code):    # redirect to framework-specific implementation
+                return True
+
+        elif key_code == 'r':
             if modifiers == 'Shift':
                 self.change_app_state('flow_path', [])  # quick jump to root (also repairing any flow path errors)
                 self.change_app_state('flow_id', '')
